@@ -2,6 +2,8 @@ import mimetypes
 from datetime import datetime
 from datetime import timedelta
 from tempfile import SpooledTemporaryFile
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
 
 from azure.core.exceptions import ResourceNotFoundError
 from azure.core.utils import parse_connection_string
@@ -17,7 +19,6 @@ from django.utils.deconstruct import deconstructible
 
 from storages.base import BaseStorage
 from storages.utils import clean_name
-from storages.utils import get_available_overwrite_name
 from storages.utils import safe_join
 from storages.utils import setting
 from storages.utils import to_bytes
@@ -118,9 +119,7 @@ class AzureStorage(BaseStorage):
     def __init__(self, **settings):
         super().__init__(**settings)
         self._service_client = None
-        self._custom_service_client = None
         self._client = None
-        self._custom_client = None
         self._user_delegation_key = None
         self._user_delegation_key_expiry = datetime.utcnow()
         if self.connection_string and (not self.account_name or not self.account_key):
@@ -155,18 +154,11 @@ class AzureStorage(BaseStorage):
             "api_version": setting("AZURE_API_VERSION", None),
         }
 
-    def _get_service_client(self, use_custom_domain):
+    def _get_service_client(self):
         if self.connection_string is not None:
             return BlobServiceClient.from_connection_string(self.connection_string)
 
-        account_domain = (
-            self.custom_domain
-            if self.custom_domain and use_custom_domain
-            else "{}.blob.{}".format(
-                self.account_name,
-                self.endpoint_suffix,
-            )
-        )
+        account_domain = "{}.blob.{}".format(self.account_name, self.endpoint_suffix)
         account_url = "{}://{}".format(self.azure_protocol, account_domain)
 
         credential = None
@@ -187,17 +179,9 @@ class AzureStorage(BaseStorage):
     @property
     def service_client(self):
         if self._service_client is None:
-            self._service_client = self._get_service_client(use_custom_domain=False)
+            self._service_client = self._get_service_client()
         return self._service_client
 
-    @property
-    def custom_service_client(self):
-        if self._custom_service_client is None:
-            self._custom_service_client = self._get_service_client(
-                use_custom_domain=True
-            )
-        return self._custom_service_client
-
     @property
     def client(self):
         if self._client is None:
@@ -206,14 +190,6 @@ class AzureStorage(BaseStorage):
             )
         return self._client
 
-    @property
-    def custom_client(self):
-        if self._custom_client is None:
-            self._custom_client = self.custom_service_client.get_container_client(
-                self.azure_container
-            )
-        return self._custom_client
-
     def get_user_delegation_key(self, expiry):
         # We'll only be able to get a user delegation key if we've authenticated with a
         # token credential.
@@ -228,10 +204,8 @@ class AzureStorage(BaseStorage):
         ):
             now = datetime.utcnow()
             key_expiry_time = now + timedelta(days=7)
-            self._user_delegation_key = (
-                self.custom_service_client.get_user_delegation_key(
-                    key_start_time=now, key_expiry_time=key_expiry_time
-                )
+            self._user_delegation_key = self.service_client.get_user_delegation_key(
+                key_start_time=now, key_expiry_time=key_expiry_time
             )
             self._user_delegation_key_expiry = key_expiry_time
 
@@ -257,17 +231,13 @@ class AzureStorage(BaseStorage):
     def _open(self, name, mode="rb"):
         return AzureStorageFile(name, mode, self)
 
-    def get_available_name(self, name, max_length=_AZURE_NAME_MAX_LEN):
-        """
-        Returns a filename that's free on the target storage system, and
-        available for new content to be written to.
-        """
-        name = clean_name(name)
-        if self.overwrite_files:
-            return get_available_overwrite_name(name, max_length)
-        return super().get_available_name(name, max_length)
-
     def exists(self, name):
+        if not name:
+            return True
+
+        if self.overwrite_files:
+            return False
+
         blob_client = self.client.get_blob_client(self._get_valid_path(name))
         return blob_client.exists()
 
@@ -306,9 +276,10 @@ class AzureStorage(BaseStorage):
         # azure expects time in UTC
         return datetime.utcnow() + timedelta(seconds=expire)
 
-    def url(self, name, expire=None, parameters=None):
+    def url(self, name, expire=None, parameters=None, mode="r"):
         name = self._get_valid_path(name)
         params = parameters or {}
+        permission = BlobSasPermissions.from_string(mode)
 
         if expire is None:
             expire = self.expiration_secs
@@ -323,13 +294,21 @@ class AzureStorage(BaseStorage):
                 name,
                 account_key=self.account_key,
                 user_delegation_key=user_delegation_key,
-                permission=BlobSasPermissions(read=True),
+                permission=permission,
                 expiry=expiry,
                 **params,
             )
             credential = sas_token
 
-        container_blob_url = self.custom_client.get_blob_client(name).url
+        container_blob_url = self.client.get_blob_client(name).url
+
+        if self.custom_domain:
+            # Replace the account name with the custom domain
+            parsed_url = urlparse(container_blob_url)
+            container_blob_url = urlunparse(
+                parsed_url._replace(netloc=self.custom_domain)
+            )
+
         return BlobClient.from_blob_url(container_blob_url, credential=credential).url
 
     def _get_content_settings_parameters(self, name, content=None):
@@ -390,16 +369,9 @@ class AzureStorage(BaseStorage):
 
     def listdir(self, path=""):
         """
-        Return directories and files for a given path.
-        Leave the path empty to list the root.
-        Order of dirs and files is undefined.
+        Return all files for a given path.
+        Given that Azure can't return paths it only returns files.
+        Works great for our little adventure.
         """
-        files = []
-        dirs = set()
-        for name in self.list_all(path):
-            n = name[len(path) :]
-            if "/" in n:
-                dirs.add(n.split("/", 1)[0])
-            else:
-                files.append(n)
-        return list(dirs), files
+
+        return [], self.list_all(path)
