@@ -19,7 +19,6 @@ from dropbox.files import UploadSessionCursor
 from dropbox.files import WriteMode
 
 from storages.base import BaseStorage
-from storages.utils import get_available_overwrite_name
 from storages.utils import setting
 
 _DEFAULT_TIMEOUT = 100
@@ -33,12 +32,6 @@ class DropboxStorageException(Exception):
 DropBoxStorageException = DropboxStorageException
 
 
-def removeprefix(prefix, name):
-    if name.startswith(prefix):
-        name = name[len(prefix) :]
-    return name
-
-
 class DropboxFile(File):
     def __init__(self, name, storage):
         self.name = name
@@ -130,6 +123,9 @@ class DropboxStorage(BaseStorage):
         self.client.files_delete(self._full_path(name))
 
     def exists(self, name):
+        if self.write_mode == "overwrite":
+            return False
+
         try:
             return bool(self.client.files_get_metadata(self._full_path(name)))
         except ApiError:
@@ -174,10 +170,7 @@ class DropboxStorage(BaseStorage):
         else:
             self._chunked_upload(content, self._full_path(name))
         content.close()
-        # .save() validates the filename isn't absolute but Dropbox requires an
-        # absolute filename.  Work with the absolute name internally but strip it
-        # off before passing up-the-chain.
-        return removeprefix(self.root_path, name).lstrip("/")
+        return name
 
     def _chunked_upload(self, content, dest_path):
         upload_session = self.client.files_upload_session_start(
@@ -199,12 +192,5 @@ class DropboxStorage(BaseStorage):
                 )
                 cursor.offset = content.tell()
 
-    def get_available_name(self, name, max_length=None):
-        """Overwrite existing file with the same name."""
-        name = self._full_path(name)
-        if self.write_mode == "overwrite":
-            return get_available_overwrite_name(name, max_length)
-        return super().get_available_name(name, max_length)
-
 
 DropBoxStorage = DropboxStorage
