@@ -4,7 +4,6 @@ import posixpath
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.core.exceptions import SuspiciousFileOperation
 from django.core.files.utils import FileProxyMixin
 from django.utils.encoding import force_bytes
 
@@ -120,25 +119,6 @@ def lookup_env(names):
             return value
 
 
-def get_available_overwrite_name(name, max_length):
-    if max_length is None or len(name) <= max_length:
-        return name
-
-    # Adapted from Django
-    dir_name, file_name = os.path.split(name)
-    file_root, file_ext = os.path.splitext(file_name)
-    truncation = len(name) - max_length
-
-    file_root = file_root[:-truncation]
-    if not file_root:
-        raise SuspiciousFileOperation(
-            'Storage tried to truncate away entire filename "%s". '
-            "Please make sure that the corresponding file field "
-            'allows sufficient "max_length".' % name
-        )
-    return os.path.join(dir_name, "{}{}".format(file_root, file_ext))
-
-
 def is_seekable(file_object):
     return not hasattr(file_object, "seekable") or file_object.seekable()
 
