@@ -2,9 +2,7 @@ import datetime
 from datetime import timedelta
 from unittest import mock
 
-import django
 from azure.storage.blob import BlobProperties
-from django.core.exceptions import SuspiciousOperation
 from django.core.files.base import ContentFile
 from django.test import TestCase
 from django.test import override_settings
@@ -17,7 +15,6 @@ class AzureStorageTest(TestCase):
     def setUp(self, *args):
         self.storage = azure_storage.AzureStorage()
         self.storage._client = mock.MagicMock()
-        self.storage._custom_client = mock.MagicMock()
         self.storage.overwrite_files = True
         self.account_name = "test"
         self.account_key = "key"
@@ -71,100 +68,38 @@ class AzureStorageTest(TestCase):
             self.storage._get_valid_path(some_path),
         )
 
-    def test_get_available_name(self):
-        self.storage.overwrite_files = False
-        client_mock = mock.MagicMock()
-        client_mock.exists.side_effect = [True, False]
-        custom_client_mock = mock.MagicMock()
-        self.storage._client.get_blob_client.return_value = client_mock
-        self.storage._custom_client.get_blob_client.return_value = custom_client_mock
-        name = self.storage.get_available_name("foo.txt")
-        self.assertTrue(name.startswith("foo_"))
-        self.assertTrue(name.endswith(".txt"))
-        self.assertTrue(len(name) > len("foo.txt"))
-        self.assertEqual(client_mock.exists.call_count, 2)
-        self.assertEqual(custom_client_mock.exists.call_count, 0)
-
-    def test_get_available_name_first(self):
-        self.storage.overwrite_files = False
-        client_mock = mock.MagicMock()
-        client_mock.exists.return_value = False
-        self.storage._client.get_blob_client.return_value = client_mock
-        self.assertEqual(
-            self.storage.get_available_name("foo bar baz.txt"), "foo bar baz.txt"
-        )
-        self.assertEqual(client_mock.exists.call_count, 1)
-
-    def test_get_available_name_max_len(self):
-        self.storage.overwrite_files = False
-        # if you wonder why this is, file-system
-        # storage will raise when file name is too long as well,
-        # the form should validate this
-        client_mock = mock.MagicMock()
-        client_mock.exists.side_effect = [True, False]
-        self.storage._client.get_blob_client.return_value = client_mock
-        self.assertRaises(ValueError, self.storage.get_available_name, "a" * 1025)
-        name = self.storage.get_available_name(
-            "a" * 1000, max_length=100
-        )  # max_len == 1024
-        self.assertEqual(len(name), 100)
-        self.assertTrue("_" in name)
-        self.assertEqual(client_mock.exists.call_count, 2)
-
-    def test_get_available_invalid(self):
-        self.storage.overwrite_files = False
-        self.storage._client.exists.return_value = False
-        if django.VERSION[:2] == (3, 0):
-            # Django 2.2.21 added this security fix:
-            # https://docs.djangoproject.com/en/3.2/releases/2.2.21/#cve-2021-31542-potential-directory-traversal-via-uploaded-files
-            # It raises SuspiciousOperation before we get to our ValueError.
-            # The fix wasn't applied to 3.0 (no longer in support), but was applied to
-            # 3.1 & 3.2.
-            self.assertRaises(ValueError, self.storage.get_available_name, "")
-            self.assertRaises(ValueError, self.storage.get_available_name, "/")
-            self.assertRaises(ValueError, self.storage.get_available_name, ".")
-            self.assertRaises(ValueError, self.storage.get_available_name, "///")
-        else:
-            self.assertRaises(SuspiciousOperation, self.storage.get_available_name, "")
-            self.assertRaises(SuspiciousOperation, self.storage.get_available_name, "/")
-            self.assertRaises(SuspiciousOperation, self.storage.get_available_name, ".")
-            self.assertRaises(
-                SuspiciousOperation, self.storage.get_available_name, "///"
-            )
-        self.assertRaises(ValueError, self.storage.get_available_name, "...")
-
     def test_url(self):
         blob_mock = mock.MagicMock()
         blob_mock.url = "https://ret_foo.blob.core.windows.net/test/some%20blob"
-        self.storage._custom_client.get_blob_client.return_value = blob_mock
+        self.storage._client.get_blob_client.return_value = blob_mock
         self.assertEqual(self.storage.url("some blob"), blob_mock.url)
-        self.storage.custom_client.get_blob_client.assert_called_once_with("some blob")
-        self.storage._client.get_blob_client.assert_not_called()
+        self.storage._client.get_blob_client.assert_called_once_with("some blob")
 
     def test_url_unsafe_chars(self):
         blob_mock = mock.MagicMock()
         blob_mock.url = "https://ret_foo.blob.core.windows.net/test/some%20blob"
-        self.storage._custom_client.get_blob_client.return_value = blob_mock
+        self.storage._client.get_blob_client.return_value = blob_mock
         self.assertEqual(
             self.storage.url("foo;?:@=&\"<>#%{}|^~[]`bar/~!*()'"), blob_mock.url
         )
-        self.storage.custom_client.get_blob_client.assert_called_once_with(
+        self.storage._client.get_blob_client.assert_called_once_with(
             "foo;?:@=&\"<>#%{}|^~[]`bar/~!*()'"
         )
-        self.storage._client.get_blob_client.assert_not_called()
 
     @mock.patch("storages.backends.azure_storage.generate_blob_sas")
     def test_url_expire(self, generate_blob_sas_mocked):
         generate_blob_sas_mocked.return_value = "foo_token"
         blob_mock = mock.MagicMock()
         blob_mock.url = "https://ret_foo.blob.core.windows.net/test/some%20blob"
-        self.storage._custom_client.get_blob_client.return_value = blob_mock
+        self.storage._client.get_blob_client.return_value = blob_mock
         self.storage.account_name = self.account_name
 
         fixed_time = make_aware(
             datetime.datetime(2016, 11, 6, 4), datetime.timezone.utc
         )
+
         with mock.patch("storages.backends.azure_storage.datetime") as d_mocked:
+            # Implicit read permission
             d_mocked.utcnow.return_value = fixed_time
             self.assertEqual(
                 self.storage.url("some blob", 100),
@@ -179,16 +114,44 @@ class AzureStorageTest(TestCase):
                 permission=mock.ANY,
                 expiry=fixed_time + timedelta(seconds=100),
             )
+            called_args, called_kwargs = generate_blob_sas_mocked.call_args
+            self.assertEqual(str(called_kwargs["permission"]), "r")
+
+            # Explicit write permission
+            d_mocked.utcnow.return_value = fixed_time
+            self.assertEqual(
+                self.storage.url("some blob", expire=100, mode="w"),
+                "https://ret_foo.blob.core.windows.net/test/some%20blob",
+            )
+            generate_blob_sas_mocked.assert_called_with(
+                self.account_name,
+                self.container_name,
+                "some blob",
+                account_key=self.account_key,
+                user_delegation_key=None,
+                permission=mock.ANY,
+                expiry=fixed_time + timedelta(seconds=100),
+            )
+            called_args, called_kwargs = generate_blob_sas_mocked.call_args
+            self.assertEqual(str(called_kwargs["permission"]), "w")
+
+    def test_url_custom_domain(self):
+        self.storage.custom_domain = "foo_domain"
+        blob_mock = mock.MagicMock()
+        blob_mock.url = "https://ret_foo.blob.core.windows.net/test/foo_name"
+        self.storage._client.get_blob_client.return_value = blob_mock
+        url = self.storage.url("foo_name")
+        self.assertEqual(url, "https://foo_domain/test/foo_name")
 
     @mock.patch("storages.backends.azure_storage.generate_blob_sas")
     def test_url_expire_user_delegation_key(self, generate_blob_sas_mocked):
         generate_blob_sas_mocked.return_value = "foo_token"
         blob_mock = mock.MagicMock()
         blob_mock.url = "https://ret_foo.blob.core.windows.net/test/some%20blob"
-        self.storage._custom_client.get_blob_client.return_value = blob_mock
+        self.storage._client.get_blob_client.return_value = blob_mock
         self.storage.account_name = self.account_name
-        custom_service_client = mock.MagicMock()
-        self.storage._custom_service_client = custom_service_client
+        service_client = mock.MagicMock()
+        self.storage._service_client = service_client
         self.storage.token_credential = "token_credential"
 
         fixed_time = make_aware(
@@ -196,9 +159,7 @@ class AzureStorageTest(TestCase):
         )
         with mock.patch("storages.backends.azure_storage.datetime") as d_mocked:
             d_mocked.utcnow.return_value = fixed_time
-            custom_service_client.get_user_delegation_key.return_value = (
-                "user delegation key"
-            )
+            service_client.get_user_delegation_key.return_value = "user delegation key"
             self.assertEqual(
                 self.storage.url("some blob", 100),
                 "https://ret_foo.blob.core.windows.net/test/some%20blob",
@@ -229,14 +190,12 @@ class AzureStorageTest(TestCase):
     def test_container_client_params_account_key(self):
         storage = azure_storage.AzureStorage()
         storage.account_name = "foo_name"
-        storage.azure_ssl = True
         storage.custom_domain = "foo_domain"
         storage.account_key = "foo_key"
         with mock.patch(
             "storages.backends.azure_storage.BlobServiceClient", autospec=True
         ) as bsc_mocked:
             client_mock = mock.MagicMock()
-            custom_client_mock = mock.MagicMock()
             bsc_mocked.return_value.get_container_client.return_value = client_mock
             self.assertEqual(storage.client, client_mock)
             bsc_mocked.assert_called_once_with(
@@ -244,40 +203,21 @@ class AzureStorageTest(TestCase):
                 credential={"account_name": "foo_name", "account_key": "foo_key"},
             )
 
-            bsc_mocked.return_value.get_container_client.return_value = (
-                custom_client_mock
-            )
-            self.assertEqual(storage.custom_client, custom_client_mock)
-            self.assertEqual(bsc_mocked.call_count, 2)
-            bsc_mocked.assert_called_with(
-                "https://foo_domain",
-                credential={"account_name": "foo_name", "account_key": "foo_key"},
-            )
-
     def test_container_client_params_sas_token(self):
         storage = azure_storage.AzureStorage()
         storage.account_name = "foo_name"
         storage.azure_ssl = False
-        storage.custom_domain = "foo_domain"
         storage.sas_token = "foo_token"
         with mock.patch(
             "storages.backends.azure_storage.BlobServiceClient", autospec=True
         ) as bsc_mocked:
             client_mock = mock.MagicMock()
-            custom_client_mock = mock.MagicMock()
             bsc_mocked.return_value.get_container_client.return_value = client_mock
             self.assertEqual(storage.client, client_mock)
             bsc_mocked.assert_called_once_with(
                 "http://foo_name.blob.core.windows.net", credential="foo_token"
             )
 
-            bsc_mocked.return_value.get_container_client.return_value = (
-                custom_client_mock
-            )
-            self.assertEqual(storage.custom_client, custom_client_mock)
-            self.assertEqual(bsc_mocked.call_count, 2)
-            bsc_mocked.assert_called_with("http://foo_domain", credential="foo_token")
-
     def test_container_client_params_token_credential(self):
         storage = azure_storage.AzureStorage()
         storage.account_name = self.account_name
@@ -334,7 +274,6 @@ class AzureStorageTest(TestCase):
             c_mocked.assert_called_once_with(
                 content_type="text/plain", content_encoding=None, cache_control=None
             )
-            self.storage._custom_client.upload_blob.assert_not_called()
 
     def test_storage_open_write(self):
         """
@@ -355,22 +294,21 @@ class AzureStorageTest(TestCase):
             timeout=20,
             overwrite=True,
         )
-        self.storage._custom_client.upload_blob.assert_not_called()
 
     def test_storage_exists(self):
-        blob_name = "blob"
-        client_mock = mock.MagicMock()
-        custom_client_mock = mock.MagicMock()
-        self.storage._client.get_blob_client.return_value = client_mock
-        self.storage._custom_client.get_blob_client.return_value = client_mock
-        self.assertTrue(self.storage.exists(blob_name))
-        self.assertEqual(client_mock.exists.call_count, 1)
-        self.assertEqual(custom_client_mock.exists.call_count, 0)
+        overwrite_files = [True, False]
+        for owf in overwrite_files:
+            self.storage.overwrite_files = owf
+            client_mock = mock.MagicMock()
+            self.storage._client.get_blob_client.return_value = client_mock
+            assert_ = self.assertFalse if owf else self.assertTrue
+            call_count = 0 if owf else 1
+            assert_(self.storage.exists("blob"))
+            self.assertEqual(client_mock.exists.call_count, call_count)
 
     def test_delete_blob(self):
         self.storage.delete("name")
         self.storage._client.delete_blob.assert_called_once_with("name", timeout=20)
-        self.storage._custom_client.delete_blob.assert_not_called()
 
     def test_storage_listdir_base(self):
         file_names = ["some/path/1.txt", "2.txt", "other/path/3.txt", "4.txt"]
@@ -386,48 +324,15 @@ class AzureStorageTest(TestCase):
         self.storage._client.list_blobs.assert_called_with(
             name_starts_with="", timeout=20
         )
-        self.storage._custom_client.list_blobs.assert_not_called()
+        self.assertEqual(len(dirs), 0)
 
-        self.assertEqual(len(dirs), 2)
-        for directory in ["some", "other"]:
-            self.assertTrue(
-                directory in dirs,
-                """ "{}" not in directory list "{}".""".format(directory, dirs),
-            )
-
-        self.assertEqual(len(files), 2)
-        for filename in ["2.txt", "4.txt"]:
+        self.assertEqual(len(files), 4)
+        for filename in ["2.txt", "4.txt", "other/path/3.txt", "some/path/1.txt"]:
             self.assertTrue(
                 filename in files,
                 """ "{}" not in file list "{}".""".format(filename, files),
             )
 
-    def test_storage_listdir_subdir(self):
-        file_names = ["some/path/1.txt", "some/2.txt"]
-
-        result = []
-        for p in file_names:
-            obj = mock.MagicMock()
-            obj.name = p
-            result.append(obj)
-        self.storage._client.list_blobs.return_value = iter(result)
-        self.storage._custom_client.list_blobs.assert_not_called()
-
-        dirs, files = self.storage.listdir("some/")
-        self.storage._client.list_blobs.assert_called_with(
-            name_starts_with="some/", timeout=20
-        )
-
-        self.assertEqual(len(dirs), 1)
-        self.assertTrue(
-            "path" in dirs, """ "path" not in directory list "{}".""".format(dirs)
-        )
-
-        self.assertEqual(len(files), 1)
-        self.assertTrue(
-            "2.txt" in files, """ "2.txt" not in files list "{}".""".format(files)
-        )
-
     def test_size_of_file(self):
         props = BlobProperties()
         props.size = 12
@@ -462,43 +367,3 @@ class AzureStorageTest(TestCase):
         self.assertEqual(storage.azure_container, "foo1")
         storage = azure_storage.AzureStorage(azure_container="foo2")
         self.assertEqual(storage.azure_container, "foo2")
-
-    @mock.patch(
-        "storages.backends.azure_storage.AzureStorage._get_service_client",
-    )
-    def test_get_service_client_use_custom_domain(self, gsc_mocked):
-        storage = azure_storage.AzureStorage()
-        storage.account_name = self.account_name
-
-        _ = storage.service_client
-        gsc_mocked.assert_called_once_with(use_custom_domain=False)
-
-        _ = storage.custom_service_client
-        gsc_mocked.assert_called_with(use_custom_domain=True)
-
-    def test_blobserviceclient_no_custom_domain(self):
-        storage = azure_storage.AzureStorage()
-        storage.account_name = "foo_name"
-        storage.custom_domain = None
-        storage.account_key = "foo_key"
-        with mock.patch(
-            "storages.backends.azure_storage.BlobServiceClient", autospec=True
-        ) as bsc_mocked:
-            client_mock = mock.MagicMock()
-            custom_client_mock = mock.MagicMock()
-            bsc_mocked.return_value.get_container_client.return_value = client_mock
-            self.assertEqual(storage.client, client_mock)
-            bsc_mocked.assert_called_once_with(
-                "https://foo_name.blob.core.windows.net",
-                credential={"account_name": "foo_name", "account_key": "foo_key"},
-            )
-
-            bsc_mocked.return_value.get_container_client.return_value = (
-                custom_client_mock
-            )
-            self.assertEqual(storage.custom_client, custom_client_mock)
-            self.assertEqual(bsc_mocked.call_count, 2)
-            bsc_mocked.assert_called_with(
-                "https://foo_name.blob.core.windows.net",
-                credential={"account_name": "foo_name", "account_key": "foo_key"},
-            )
