@@ -1,15 +1,7 @@
 import io
-import sys
 from datetime import datetime
-from unittest import SkipTest
 from unittest import mock
 
-if sys.version_info >= (3, 12, 0):
-    raise SkipTest(
-        "dropbox library does not support Python 3.12+. "
-        "Skipping all tests in test_dropbox.py"
-    )
-
 from django.core.exceptions import ImproperlyConfigured
 from django.core.exceptions import SuspiciousFileOperation
 from django.core.files.base import File
@@ -100,6 +92,11 @@ class DropboxTest(TestCase):
         exists = self.storage.exists("bar")
         self.assertFalse(exists)
 
+    @mock.patch("dropbox.Dropbox.files_get_metadata", return_value=[FILE_METADATA_MOCK])
+    def test_exists_overwrite_mode(self, *args):
+        self.storage.write_mode = "overwrite"
+        self.assertFalse(self.storage.exists("foo"))
+
     @mock.patch("dropbox.Dropbox.files_list_folder", return_value=FILES_MOCK)
     def test_listdir(self, *args):
         dirs, files = self.storage.listdir("/")
