@@ -11,6 +11,7 @@ from urllib.parse import urlparse
 
 import boto3
 import boto3.s3.transfer
+import botocore
 from botocore.config import Config as ClientConfig
 from botocore.exceptions import ClientError
 from django.conf import settings
@@ -35,25 +36,37 @@ class S3StorageTests(TestCase):
     def setUp(self):
         self.storage = s3.S3Storage()
         self.storage._connections.connection = mock.MagicMock()
+        self.storage._unsigned_connections.connection = mock.MagicMock()
 
-    def test_s3_session(self):
+    @mock.patch("boto3.Session")
+    def test_s3_session(self, session):
         with override_settings(AWS_S3_SESSION_PROFILE="test_profile"):
-            with mock.patch("boto3.Session") as mock_session:
-                storage = s3.S3Storage()
-                _ = storage.connection
-                mock_session.assert_called_once_with(profile_name="test_profile")
+            storage = s3.S3Storage()
+            _ = storage.connection
+            session.assert_called_once_with(profile_name="test_profile")
 
-    def test_client_config(self):
+    @mock.patch("boto3.Session.resource")
+    def test_client_config(self, resource):
         with override_settings(
             AWS_S3_CLIENT_CONFIG=ClientConfig(max_pool_connections=30)
         ):
             storage = s3.S3Storage()
-            with mock.patch("boto3.Session.resource") as mock_resource:
-                _ = storage.connection
-                mock_resource.assert_called_once()
-                self.assertEqual(
-                    30, mock_resource.call_args[1]["config"].max_pool_connections
-                )
+            _ = storage.connection
+            resource.assert_called_once()
+            self.assertEqual(30, resource.call_args[1]["config"].max_pool_connections)
+
+    @mock.patch("boto3.Session.resource")
+    def test_connection_unsiged(self, resource):
+        with override_settings(AWS_S3_ADDRESSING_STYLE="virtual"):
+            storage = s3.S3Storage()
+            _ = storage.unsigned_connection
+            resource.assert_called_once()
+            self.assertEqual(
+                botocore.UNSIGNED, resource.call_args[1]["config"].signature_version
+            )
+            self.assertEqual(
+                "virtual", resource.call_args[1]["config"].s3["addressing_style"]
+            )
 
     def test_pickle_with_bucket(self):
         """
@@ -317,23 +330,6 @@ class S3StorageTests(TestCase):
             content_str = file.read()
             self.assertEqual(content_str, "")
 
-    def test_storage_open_readlines(self):
-        """
-        Test readlines with file opened in "r" and "rb" modes
-        """
-        name = "test_open_readlines_string.txt"
-        with io.BytesIO() as temp_file:
-            temp_file.write(b"line1\nline2")
-            with self.storage.open(name, "r") as f1:
-                f1._file = temp_file
-                content_lines = f1.readlines()
-                self.assertEqual(content_lines, ["line1\n", "line2"])
-                temp_file.seek(0)
-                with self.storage.open(name, "rb") as f2:
-                    f2._file = temp_file
-                    content_lines = f2.readlines()
-                    self.assertEqual(content_lines, [b"line1\n", b"line2"])
-
     def test_storage_open_write(self):
         """
         Test opening a file in write mode
@@ -348,7 +344,7 @@ class S3StorageTests(TestCase):
             "ACL": "public-read",
         }
 
-        with self.storage.open(name, "w") as file:
+        with self.storage.open(name, "wb") as file:
             self.storage.bucket.Object.assert_called_with(name)
             obj = self.storage.bucket.Object.return_value
             # Set the name of the mock object
@@ -398,7 +394,7 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        with self.storage.open(name, "w"):
+        with self.storage.open(name, "wb"):
             self.storage.bucket.Object.assert_called_with(name)
             obj = self.storage.bucket.Object.return_value
             obj.load.side_effect = ClientError(
@@ -429,7 +425,7 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        with self.storage.open(name, "w"):
+        with self.storage.open(name, "wb"):
             self.storage.bucket.Object.assert_called_with(name)
             obj = self.storage.bucket.Object.return_value
 
@@ -451,7 +447,7 @@ class S3StorageTests(TestCase):
             "StorageClass": "REDUCED_REDUNDANCY",
         }
 
-        with self.storage.open(name, "w") as file:
+        with self.storage.open(name, "wb") as file:
             self.storage.bucket.Object.assert_called_with(name)
             obj = self.storage.bucket.Object.return_value
             # Set the name of the mock object
@@ -500,6 +496,7 @@ class S3StorageTests(TestCase):
         )
 
     def test_storage_exists(self):
+        self.storage.file_overwrite = False
         self.assertTrue(self.storage.exists("file.txt"))
         self.storage.connection.meta.client.head_object.assert_called_with(
             Bucket=self.storage.bucket_name,
@@ -507,6 +504,7 @@ class S3StorageTests(TestCase):
         )
 
     def test_storage_exists_false(self):
+        self.storage.file_overwrite = False
         self.storage.connection.meta.client.head_object.side_effect = ClientError(
             {"Error": {}, "ResponseMetadata": {"HTTPStatusCode": 404}},
             "HeadObject",
@@ -518,6 +516,7 @@ class S3StorageTests(TestCase):
         )
 
     def test_storage_exists_other_error_reraise(self):
+        self.storage.file_overwrite = False
         self.storage.connection.meta.client.head_object.side_effect = ClientError(
             {"Error": {}, "ResponseMetadata": {"HTTPStatusCode": 403}},
             "HeadObject",
@@ -529,6 +528,10 @@ class S3StorageTests(TestCase):
             cm.exception.response["ResponseMetadata"]["HTTPStatusCode"], 403
         )
 
+    def test_storage_exists_overwrite(self):
+        self.storage.file_overwrite = True
+        self.assertFalse(self.storage.exists("foo"))
+
     def test_storage_delete(self):
         self.storage.delete("path/to/file.txt")
         self.storage.bucket.Object.assert_called_with("path/to/file.txt")
@@ -681,10 +684,10 @@ class S3StorageTests(TestCase):
     def test_storage_url(self):
         name = "test_storage_size.txt"
         url = "http://aws.amazon.com/%s" % name
-        self.storage.bucket.meta.client.generate_presigned_url.return_value = url
+        self.storage.connection.meta.client.generate_presigned_url.return_value = url
         self.storage.bucket.name = "bucket"
         self.assertEqual(self.storage.url(name), url)
-        self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
+        self.storage.connection.meta.client.generate_presigned_url.assert_called_with(
             "get_object",
             Params={"Bucket": self.storage.bucket.name, "Key": name},
             ExpiresIn=self.storage.querystring_expire,
@@ -692,9 +695,8 @@ class S3StorageTests(TestCase):
         )
 
         custom_expire = 123
-
         self.assertEqual(self.storage.url(name, expire=custom_expire), url)
-        self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
+        self.storage.connection.meta.client.generate_presigned_url.assert_called_with(
             "get_object",
             Params={"Bucket": self.storage.bucket.name, "Key": name},
             ExpiresIn=custom_expire,
@@ -702,16 +704,21 @@ class S3StorageTests(TestCase):
         )
 
         custom_method = "HEAD"
-
         self.assertEqual(self.storage.url(name, http_method=custom_method), url)
-        self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
+        self.storage.connection.meta.client.generate_presigned_url.assert_called_with(
             "get_object",
             Params={"Bucket": self.storage.bucket.name, "Key": name},
             ExpiresIn=self.storage.querystring_expire,
             HttpMethod=custom_method,
         )
 
-    def test_storage_url_custom_domain_signed_urls(self):
+    def test_url_unsigned(self):
+        self.storage.querystring_auth = False
+        self.storage.url("test_name")
+        self.storage.unsigned_connection.meta.client.generate_presigned_url.assert_called_once()
+
+    @mock.patch("storages.backends.s3.datetime")
+    def test_storage_url_custom_domain_signed_urls(self, dt):
         key_id = "test-key"
         filename = "file.txt"
         pem = dedent(
@@ -749,11 +756,8 @@ class S3StorageTests(TestCase):
             self.assertEqual(self.storage.url(filename), url)
 
             self.storage.querystring_auth = True
-            with mock.patch("storages.backends.s3.datetime") as mock_datetime:
-                mock_datetime.utcnow.return_value = datetime.datetime.utcfromtimestamp(
-                    0
-                )
-                self.assertEqual(self.storage.url(filename), signed_url)
+            dt.utcnow.return_value = datetime.datetime.utcfromtimestamp(0)
+            self.assertEqual(self.storage.url(filename), signed_url)
 
     def test_generated_url_is_encoded(self):
         self.storage.custom_domain = "mock.cloudfront.net"
@@ -783,21 +787,6 @@ class S3StorageTests(TestCase):
         self.assertEqual(parsed_url.path, "/filename.mp4")
         self.assertEqual(parsed_url.query, "version=10")
 
-    def test_strip_signing_parameters(self):
-        expected = "http://bucket.s3-aws-region.amazonaws.com/foo/bar"
-        self.assertEqual(
-            self.storage._strip_signing_parameters(
-                "%s?X-Amz-Date=12345678&X-Amz-Signature=Signature" % expected
-            ),
-            expected,
-        )
-        self.assertEqual(
-            self.storage._strip_signing_parameters(
-                "%s?expires=12345678&signature=Signature" % expected
-            ),
-            expected,
-        )
-
     @skipIf(threading is None, "Test requires threading")
     def test_connection_threading(self):
         connections = []
@@ -982,6 +971,29 @@ class S3StorageTests(TestCase):
                 access_key="foo", secret_key="boo", session_profile="moo"
             )
 
+    def test_security_token(self):
+        with override_settings(AWS_SESSION_TOKEN="baz"):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.security_token, "baz")
+
+        with override_settings(AWS_SECURITY_TOKEN="baz"):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.security_token, "baz")
+
+        with mock.patch.dict(
+            os.environ,
+            {"AWS_SESSION_TOKEN": "baz"},
+        ):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.security_token, "baz")
+
+        with mock.patch.dict(
+            os.environ,
+            {"AWS_SECURITY_TOKEN": "baz"},
+        ):
+            storage = s3.S3Storage()
+            self.assertEqual(storage.security_token, "baz")
+
 
 class S3StaticStorageTests(TestCase):
     def setUp(self):
@@ -1144,6 +1156,41 @@ class S3StorageTestsWithMoto(TestCase):
             s3.S3Storage.default_content_type,
         )
 
+    def test_storage_open_reading_with_newlines(self):
+        """Test file reading with "r" and "rb" and various newline characters."""
+        name = "test_storage_open_read_with_newlines.txt"
+        with io.BytesIO() as temp_file:
+            temp_file.write(b"line1\nline2\r\nmore\rtext\n")
+            self.storage.save(name, temp_file)
+            file = self.storage.open(name, "r")
+            content_str = file.read()
+            file.close()
+        self.assertEqual(content_str, "line1\nline2\nmore\ntext\n")
+
+        with io.BytesIO() as temp_file:
+            temp_file.write(b"line1\nline2\r\nmore\rtext\n")
+            self.storage.save(name, temp_file)
+            file = self.storage.open(name, "rb")
+            content_str = file.read()
+            file.close()
+        self.assertEqual(content_str, b"line1\nline2\r\nmore\rtext\n")
+
+        with io.BytesIO() as temp_file:
+            temp_file.write(b"line1\nline2\r\nmore\rtext")
+            self.storage.save(name, temp_file)
+            file = self.storage.open(name, "r")
+            content_lines = file.readlines()
+            file.close()
+        self.assertEqual(content_lines, ["line1\n", "line2\n", "more\n", "text"])
+
+        with io.BytesIO() as temp_file:
+            temp_file.write(b"line1\nline2\r\nmore\rtext")
+            self.storage.save(name, temp_file)
+            file = self.storage.open(name, "rb")
+            content_lines = file.readlines()
+            file.close()
+        self.assertEqual(content_lines, [b"line1\n", b"line2\r\n", b"more\r", b"text"])
+
 
 class TestBackwardsNames(TestCase):
     def test_importing(self):
